@@ -1,6 +1,6 @@
 package HTML::Template;
 
-$HTML::Template::VERSION = '1.2.1';
+$HTML::Template::VERSION = '1.1';
 
 =head1 NAME
 
@@ -31,10 +31,8 @@ Now create a small CGI program:
   my $template = HTML::Template->new(filename => 'test.tmpl');
 
   # fill in some parameters
-  $template->param(
-      HOME => $ENV{HOME},
-      PATH => $ENV{PATH},
-  );
+  $template->param('HOME', $ENV{HOME});
+  $template->param('PATH', $ENV{PATH});
 
   # send the obligatory Content-Type
   print "Content-Type: text/html\n\n";
@@ -109,18 +107,6 @@ work as planned.
 The "NAME=" in the tag is optional, although for extensibility's sake I
 recommend using it.  Example - "<TMPL_LOOP LOOP_NAME>" is acceptable.
 
-If you're a fanatic about valid HTML and would like your templates
-to conform to valid HTML syntax, you may optionally type template tags
-in the form of HTML comments. This may be of use to HTML authors who
-would like to validate their templates' HTML syntax prior to
-HTML::Template processing, or who use DTD-savvy editing tools.
-
-  <!-- TMPL_VAR NAME=PARAM1 -->
-
-In order to realize a dramatic savings in bandwidth, the standard
-(non-comment) tags will be used throughout the rest of this
-documentation.
-
 =head2 <TMPL_VAR ?ESCAPE=1? NAME="PARAMETER_NAME">
 
 The <TMPL_VAR> tag is very simple.  For each <TMPL_VAR> tag in the
@@ -270,6 +256,8 @@ tried next.  Next, the "path" new() option is consulted.  As a final
 attempt, the filename is passed to open() directly.  See below for
 more information on HTML_TEMPLATE_ROOT and the "path" option to new().
 
+NOTE: Currently, each <TMPL_INCLUDE> must be on a seperate line by itself.
+
 =head2 <TMPL_IF NAME="CONTROL_PARAMETER_NAME"> </TMPL_IF>
 
 The <TMPL_IF> tag allows you to include or not include a block of the
@@ -444,6 +432,13 @@ considered obsolete.
 
 =item *
 
+no_includes - if you know that your template does not have
+TMPL_INCLUDE tags, then you can set no_includes to 1.  This will give
+a small performance gain, since the prepass for include tags can be
+skipped.  Defaults to 0.
+
+=item *
+
 path - you can set this variable with a list of paths to search for
 files specified with the "filename" option to new() and for files
 included with the <TMPL_INCLUDE> tag.  This list is only consulted
@@ -463,9 +458,8 @@ Example:
 
 vanguard_compatibility_mode - if set to 1 the module will expect to
 see <TMPL_VAR>s that look like %NAME% instead of the standard syntax.
-Also sets die_on_bad_params => 0.  If you're not at Vanguard Media
-trying to use an old format template don't worry about this one.
-Defaults to 0.
+If you're not at Vanguard Media trying to use an old format template
+don't worry about this one.  Defaults to 0.
 
 =item *
 
@@ -525,6 +519,7 @@ sub new {
                cache_debug => 0,
                die_on_bad_params => 1,
                vanguard_compatibility_mode => 0,
+               no_includes => 0,
                associate => [],
                path => []
               );
@@ -538,10 +533,6 @@ sub new {
   # blind_cache = 1 implies cache = 1
   $options->{blind_cache} and $options->{cache} = 1;
 
-  # vanguard_compatibility_mode implies die_on_bad_params = 1
-  $options->{vanguard_compatibility_mode} and 
-    $options->{die_on_bad_params} = 1;
-
   # handle the "type", "source" parameter format (does anyone use it?)
   if (exists($options->{type})) {
     exists($options->{source}) or die "HTML::Template->new() called with 'type' parameter set, but no 'source'!";
@@ -577,7 +568,7 @@ sub new {
   exists($options->{arrayref}) and $source_count++;
   exists($options->{scalarref}) and $source_count++;
   if ($source_count != 1) {
-    die "HTML::Template->new called with multiple (or no) template sources specified!  A valid call to new() has exactly one filename => 'file' OR exactly one scalarRef => \\\$scalar OR exactly one arrayRef => \\\@array";    
+    die "HTML::Template->new called with multiple (or no) template sources specified!  A valid call to new() has exactly one filename => 'file' OR exactly one scalarRef => \\\$scalar OR exactly one arrayRef = \\\@array";    
   }
 
   # initialize data structures
@@ -616,7 +607,7 @@ sub _new_from_loop {
   delete($options->{param_map});
   delete($options->{parse_stack});
 
- return $self;
+  return $self;
 }
 
 # a few shortcuts to new(), of possible use...
@@ -780,10 +771,9 @@ sub _init_template {
 
       die "HTML::Template->new() : Cannot open template file $filename - file does not exist or is unreadable.";
     }
-
-    # we'll need this for future reference - to call stat() for example.
-    $options->{filename} = $filepath;
     
+    $options->{filename} = $filepath;
+
     # read into the array
     my @templateArray = <TEMPLATE>;
     close(TEMPLATE);
@@ -872,10 +862,9 @@ sub _parse {
       # a general regex to match any and all TMPL_* tags
 
       if ($line =~ /
-                    (.*?<)
-                    (?:!--\s*)?
+                    (.*?)
                     (
-                      \/?[tT][mM][pP][lL]_
+                      <\/?[tT][mM][pP][lL]_
                       (?:
                          (?:[Vv][Aa][Rr])
                          |
@@ -911,17 +900,16 @@ sub _parse {
                     )?
                     "?
                     \s*
-                    (?:--)?>
+                    >
                     (.*)
                    /sgx) {
         my $pre = $1; # what comes before
-        chop $pre; # remove trailing <
-        my $which = uc($2); # which tag is it
+        my $which = $2; # which tag is it
         my $escape = $3;
         my $name = lc $4; # what name for the tag?  undef for a /tag
         my $post = $5; # what comes after on the line
 
-        if ($which eq 'TMPL_VAR') {
+        if ($which =~ /^<[tT][mM][pP][lL]_[Vv][Aa][Rr]/) {
           $options->{debug} and print STDERR "### HTML::Template Debug ### $line_number : parsed VAR $name\n";
           
           # if we already have this var, then simply link to the existing
@@ -955,7 +943,7 @@ sub _parse {
           $line = $post;         
           next PASS;          
         
-        } elsif ($which eq 'TMPL_LOOP') {
+        } elsif ($which =~ /^<[tT][mM][pP][lL]_[Ll][Oo][Oo][Pp]/) {
          # we've got a loop start
           $options->{debug} and print STDERR "### HTML::Template Debug ### $line_number : LOOP $name start\n";
 
@@ -1002,7 +990,7 @@ sub _parse {
           $line = $post;
           next PASS;
 
-        } elsif ($which eq '/TMPL_LOOP') {
+        } elsif ($which =~ /^<\/[tT][mM][pP][lL]_[Ll][Oo][Oo][Pp]/) {
           $options->{debug} and print STDERR "### HTML::Template Debug ### $line_number : LOOP end\n";
 
           defined $escape and 
@@ -1048,7 +1036,7 @@ sub _parse {
           $line = $post;
           next PASS;
           
-        } elsif ($which eq 'TMPL_IF') {
+        } elsif ($which =~ /^<[Tt][Mm][Pp][Ll]_[Ii][Ff]/) {
           $options->{debug} and print STDERR "### HTML::Template Debug ### $line_number : IF $name start\n";
 
           defined($name) or die "HTML::Template->new() : found TMPL_IF with no name at $line_number!";
@@ -1086,14 +1074,14 @@ sub _parse {
           $line = $post;
           next PASS;
 
-        } elsif ($which eq '/TMPL_IF') {
+        } elsif ($which =~ /^<\/[Tt][Mm][Pp][Ll]_[Ii][Ff]/x) {
           $options->{debug} and print STDERR "### HTML::Template Debug ###$line_number : IF end\n";
 
           defined $escape and 
             die "HTML::Template->new() : Found ESCAPE option in a /TMPL_IF tag at line $line_number.  ESCAPE is only valid for TMPL_VARs.";
 
           my $if = pop(@ifstack);
-          die "HTML::Template->new() : found </TMPL_IF> with no matching <TMPL_IF|ELSE> at $line_number!" unless defined $if;
+          die "HTML::Template->new() : found </TMPL_IF|ELSE> with no matching <TMPL_IF|ELSE> at $line_number!" unless defined $if;
 
           # push text coming before the tag onto the pstack,
           # concatenating with preceding text if possible.
@@ -1113,7 +1101,7 @@ sub _parse {
           $line = $post;
           next PASS;
 
-        } elsif ($which eq 'TMPL_ELSE') {
+        } elsif ($which =~ /^<[Tt][Mm][Pp][Ll]_[Ee][Ll][Ss][Ee]/) {
           $options->{debug} and print STDERR "### HTML::Template Debug ### $line_number : ELSE\n";
 
           defined $escape and 
@@ -1144,7 +1132,7 @@ sub _parse {
           $line = $post;
           next PASS;
 
-        } elsif ($which eq 'TMPL_INCLUDE') {
+        } elsif ($which =~ /^<[Tt][Mm][Pp][Ll]_[Ii][Nn][Cc][Ll][Uu][Dd][Ee]/) {
           # handle TMPL_INCLUDEs
           $options->{debug} and print STDERR "### HTML::Template Debug ### $line_number : INCLUDE $name \n";
 
@@ -1266,7 +1254,7 @@ sub _parse {
 
 =head2 param
 
-param() can be called in a number of ways
+param() can be called in three ways
 
 
 1) To return a list of parameters in the template : 
@@ -1293,69 +1281,21 @@ param() can be called in a number of ways
                    ]
                   );
 
-4) To set the value of a a number of parameters :
-
-     # For simple TMPL_VARs:
-     $self->param(PARAM => 'value', 
-                  PARAM2 => 'value'
-                 );
-
-      # And with some TMPL_LOOPs:
-      $self->param(PARAM => 'value', 
-                   PARAM2 => 'value',
-                   LOOP_PARAM => 
-                   [ 
-                    { PARAM => VALUE_FOR_FIRST_PASS, ... }, 
-                    { PARAM => VALUE_FOR_SECOND_PASS, ... } 
-                    ...
-                   ],
-                   ANOTHER_LOOP_PARAM => 
-                   [ 
-                    { PARAM => VALUE_FOR_FIRST_PASS, ... }, 
-                    { PARAM => VALUE_FOR_SECOND_PASS, ... } 
-                    ...
-                   ]
-                  );
-
-5) To set the value of a a number of parameters using a hash-ref :
-
-      $self->param(
-                   { 
-                      PARAM => 'value', 
-                      PARAM2 => 'value',
-                      LOOP_PARAM => 
-                      [ 
-                        { PARAM => VALUE_FOR_FIRST_PASS, ... }, 
-                        { PARAM => VALUE_FOR_SECOND_PASS, ... } 
-                        ...
-                      ],
-                      ANOTHER_LOOP_PARAM => 
-                      [ 
-                        { PARAM => VALUE_FOR_FIRST_PASS, ... }, 
-                        { PARAM => VALUE_FOR_SECOND_PASS, ... } 
-                        ...
-                      ]
-                    }
-                   );
-
 =cut
 
 
 sub param {
-  my $self = shift;
+  my ($self, $param, $value) = @_;
   my $options = $self->{options};
   my $param_map = $self->{param_map};
+  my $type;
 
-  # the no-parameter case - return list of parameters in the template.
-  return keys(%{$param_map}) unless scalar(@_);
-  
-  my $first = shift;
-  my $type = ref $first;
+  if (!defined($param)) {
+    # return a list of parameters in this template
+    return keys(%{$param_map});
 
-  # the one-parameter case - could be a parameter value request or a
-  # hash-ref.
-  if (!scalar(@_) and !length($type)) {
-    my $param = lc $first;
+  } elsif (!defined($value)) {
+    $param = lc($param);
     
     # check for parameter existence 
     $options->{die_on_bad_params} and !exists($param_map->{$param}) and
@@ -1370,45 +1310,29 @@ sub param {
     ($type eq 'HTML::Template::LOOP') and
       return $param_map->{$param}[HTML::Template::LOOP::PARAM_SET];
     die "Unknown param type $type!";
-  } 
-
-  if (!scalar(@_)) {
-    die "HTML::Template->param() : Single reference arg to param() must be a hash-ref!  You gave me a $type." 
-      unless $type eq 'HASH';  
-    push(@_, %{$first});
   } else {
-    unshift(@_, $first);
-
-  }
-  
-  die "HTML::Template->param() : You gave me an odd number of parameters to param()!  Read the docs, man." 
-    unless ((@_ % 2) == 0);
-
-  for (my $x = 0; $x <= $#_; $x += 2) {
-    my $param = lc $_[$x];
-    my $value = $_[($x + 1)];
-
+    $param = lc($param);
     # check that this param exists in the template
     $options->{die_on_bad_params} and !exists($param_map->{$param}) and
       die("HTML::Template : Attempt to set nonexistent parameter $param : (die_on_bad_params => 1)");
     
-    # if we're not going to do from bad param names, we need to ignore
-    # them...
-    next unless (exists($param_map->{$param}));
+    return unless (exists($param_map->{$param}) and
+                   defined($param_map->{$param}));
     
     # copy in contents of ARRAY refs to prevent confusion - 
     # thanks Richard!
+    $type = ref($param_map->{$param});
     if ( ref($value) eq 'ARRAY' ) {
-      (ref($param_map->{$param}) eq 'HTML::Template::LOOP') or
+      ($type eq 'HTML::Template::LOOP') or
         die "HTML::Template::param() : attempt to set parameter $param with an array ref - parameter is not a TMPL_LOOP!";
       $param_map->{$param}[HTML::Template::LOOP::PARAM_SET] = [@{$value}];
     } else {
-      (ref($param_map->{$param}) eq 'HTML::Template::VAR') or
+      ($type eq 'HTML::Template::VAR') or
         die "HTML::Template::param() : attempt to set parameter $param with a scalar - parameter is not a TMPL_VAR!";
       ${$param_map->{$param}} = $value;
     }
+    return 1;
   }
-  return 1;
 }
 
 =head2 clear_params()
@@ -1459,6 +1383,7 @@ important for the internal implementation of loops.
 =cut
 
 
+
 sub output {
   my $self = shift;
   my $options = $self->{options};
@@ -1493,8 +1418,7 @@ sub output {
   *parse_stack = $self->{parse_stack};
   my $result = '';
   my $type;
-  my $parse_stack_length = $#parse_stack;
-  for (my $x = 0; $x <= $parse_stack_length; $x++) {
+  for (my $x = 0; $x <= $#parse_stack; $x++) {
     *line = \$parse_stack[$x];
     $type = ref($line);
 
@@ -1582,7 +1506,9 @@ sub output {
   
   my $result = '';
   foreach my $value_set (@$value_sets_array) {
-    $template->param($value_set);
+    foreach my $name (keys %$value_set) {
+      $template->param($name, $value_set->{$name});
+    }
     $result .= $template->output;
     $template->clear_params;
   }
@@ -1757,7 +1683,6 @@ provided by:
    James William Carlson
    Frank D. Cringle
    Winfried Koenig
-   Matthew Wickline
 
 Thanks!
 
